import { Injectable } from '@angular/core';
import { TesService } from './tes.service';
import { ComputesitesService } from './computesites.service';
import { AuthorisationService } from './authorisation.service';
import { Job } from './job';
import { Identity } from './identity';
import { Subscription, interval, pipe, Observable } from 'rxjs';
import { BehaviorSubject, timer } from 'rxjs';
import { repeat } from 'rxjs/operators';
import { NotificationsService } from './notifications.service';


@Injectable({
  providedIn: 'root'
})
export class JobsService {
  public jobs$: BehaviorSubject<Job[]>;
  private tsub: Subscription;
  private id: Identity;

  constructor(private tes: TesService, private authService: AuthorisationService, private notifications: NotificationsService ) { 
    this.jobs$ = new BehaviorSubject([]);
  }

  public setId(id: Identity) {
    this.id = id;
    if (this.tsub !== undefined) {
      this.tsub.unsubscribe()
    }
    this.tsub = timer(5000).pipe(repeat()).subscribe(() => this.refreshJobs())
  }

  public refreshJobs() {
    if (this.id !== undefined && this.id !== null) {
      var query$: Observable<Job[]>;
      query$ = this.tes.runCommand(this.id, this.id.site.statcmd)
      query$.subscribe((qjobs) => this.jobs$.next(<Job[]>qjobs),
                       (error) => this.getJobsError(error,this.id))
    }

  }

  public getJobsError(error,identity: Identity) {
    this.tsub.unsubscribe();
    if (error.hasOwnProperty("error") && error.error.hasOwnProperty("message")) {
      if (error.error.message.indexOf("Permission denied") != -1) {
        this.notifications.notify("Your login appears to have expired. Please log in again");
        this.authService.updateAgentContents().subscribe((_) => {return});
        return;
      } 
    }
    this.notifications.notify("Unable to retrieve a list of running jobs.\nDid your session expire?")
    console.error(error);
    this.authService.updateAgentContents().subscribe((_) => {return});
  }

}
